'use strict';

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var _regeneratorRuntime = _interopDefault(require('@babel/runtime/regenerator'));
var _extends = _interopDefault(require('@babel/runtime/helpers/extends'));
var _asyncToGenerator = _interopDefault(require('@babel/runtime/helpers/asyncToGenerator'));
var _JSON$stringify = _interopDefault(require('@babel/runtime/core-js/json/stringify'));
var rollupPluginutils = require('rollup-pluginutils');
var compiler = require('vue-component-compiler');
var path = require('path');
var logger = _interopDefault(require('debug'));

var debug = logger('rollup-plugin-vue');

function s(any) {
  return _JSON$stringify(any, null, 2);
}

function relative$1(p) {
  if (p.startsWith(process.cwd())) {
    p = p.replace(process.cwd(), '');
    p = p.replace(/^[\\/]/, '');
  }

  return p;
}

function inline(src) {
  return "import target from '".concat(src, "'\nexport default target");
}

function inlineStyle(_ref, descriptor, _ref2) {
  var id = _ref.id,
      index = _ref.index,
      code = _ref.code,
      map = _ref.map,
      modules = _ref.modules;
  var isServer = _ref2.isServer,
      isProduction = _ref2.isProduction,
      inject = _ref2.inject;
  var output = "const cssModules = ".concat(modules ? s(modules) : '{}', "\n");
  output += "export const style = ".concat(s(code), "\n") + "export const map = ".concat(s(map), "\n") + "const content = [[".concat(index, ", style, ").concat(descriptor.attrs.media && s(descriptor.attrs.media), ", ").concat(isProduction ? 'undefined' : 'map', " ]]\n");

  if (inject) {
    if (isServer) {
      output += "import inject from 'vue-component-compiler/src/runtime/inject-style-server'\n" + "cssModules.__inject__ = context => inject(".concat(s(id), ", content, ").concat(isProduction, ", context)\n");
    } else {
      output += "import inject from 'vue-component-compiler/src/runtime/inject-style-client'\n" + "cssModules.__inject__ = () => inject(".concat(s(id), ", content, ").concat(isProduction, ")\n");
    }
  }

  output += "export default cssModules";
  return output;
}

function makeId(id, type, lang, index) {
  return id + '?' + _JSON$stringify({
    type: type,
    index: index
  }) + '#.' + lang;
}

function vue() {
  var opts = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  debug('Yo! rolling vue!');
  var filter = rollupPluginutils.createFilter(opts.include, opts.exclude);
  var cache = {};
  delete opts.include;
  delete opts.exclude;
  var vueOptions = {
    hasStyleInjectFn: true,
    isServer: opts.server === true || process.env.VUE_ENV === 'server',
    isProduction: opts.production === true || process.env.NODE_ENV === 'production'
  };
  var cwd = process.cwd();

  function parseId(id) {
    var relative$$1 = path.relative(cwd, id);

    if (/\.vue\?\{[^#]+\}#\./.test(id)) {
      var start = id.lastIndexOf('?');
      var end = id.lastIndexOf('#');

      try {
        var parsed = JSON.parse(id.substring(start + 1, end));
        parsed.id = id.substring(0, start);
        parsed.relative = relative$$1;
        return parsed;
      } catch (e) {
        console.error(e);
      }
    }

    if (!filter(id) || id.endsWith('.vue')) return {
      sfc: true,
      id: id,
      relative: relative$$1
    };
    return {
      ignore: true,
      id: id,
      relative: relative$$1
    };
  }

  return {
    name: 'vue',
    resolveId: function resolveId(request) {
      var q = parseId(request);
      if (q.ignore || q.sfc) return;
      debug("  > Resolve --> ".concat(q.relative));
      var descriptor = cache[q.id];
      if (!descriptor) throw Error("SFC (".concat(q.id, ") is not processed."));
      var part = descriptor[q.type] || (q.type === 'style' ? descriptor.styles[q.index] : descriptor.customBlocks[q.index]);

      if (part && part.src) {
        return part.src;
      }

      return request;
    },
    load: function load(request) {
      var q = parseId(request);
      if (q.ignore || q.sfc) return;
      debug("  > Load    --> ".concat(q.relative));
      var id = q.id;
      var filename = relative$1(id);
      var descriptor = cache[id];
      if (!descriptor) throw Error("SFC (".concat(filename, ") is not processed."));

      switch (q.type) {
        case 'template':
          if (!descriptor.template) return '';
          if (descriptor.template.src) return;
          return {
            code: descriptor.template.content,
            map: descriptor.template.map
          };

        case 'script':
          if (!descriptor.script) return {
            code: 'export default {}\n',
            map: {
              mappings: ''
            }
          };
          return {
            code: descriptor.script.src ? inline(descriptor.script.src) : descriptor.script.content,
            map: descriptor.script.map
          };

        case 'style':
          var style = descriptor.styles[q.index];
          if (style.src) return;
          return {
            code: style.content,
            map: style.map
          };

        case 'custom':
          var block = descriptor.customBlocks[q.index];
          if (block.src) return;
          return block.content;
      }
    },
    transform: function () {
      var _transform = _asyncToGenerator(
      /*#__PURE__*/
      _regeneratorRuntime.mark(function _callee(source, request) {
        var q, filename, scopeId, descriptor, code, output, style, result;
        return _regeneratorRuntime.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                q = parseId(request);

                if (!(q.ignore || q.dep)) {
                  _context.next = 3;
                  break;
                }

                return _context.abrupt("return");

              case 3:
                filename = relative$1(q.id);
                scopeId = compiler.generateScopeId(filename, source, vueOptions.isProduction);

                if (!q.sfc) {
                  _context.next = 11;
                  break;
                }

                debug(">> Transform: ".concat(q.relative));
                descriptor = compiler.parse(source, filename);
                cache[q.id] = descriptor;
                code = compiler.assemble({
                  script: {
                    id: descriptor.script && makeId(q.id, 'script', descriptor.script.lang || 'js'),
                    descriptor: descriptor.script
                  },
                  render: {
                    id: descriptor.template && makeId(q.id, 'template', descriptor.template.lang || 'html'),
                    descriptor: descriptor.template
                  },
                  styles: descriptor.styles.map(function (style, index) {
                    return {
                      id: makeId(q.id, 'style', style.lang || 'css', index),
                      descriptor: style
                    };
                  }),
                  customBlocks: vueOptions.isProduction ? [] : descriptor.customBlocks.map(function (block, index) {
                    return {
                      id: makeId(q.id, 'custom', block.lang || block.type),
                      descriptor: block
                    };
                  })
                }, filename, _extends({}, vueOptions, {
                  scopeId: scopeId,
                  moduleIdentifier: scopeId
                }));
                return _context.abrupt("return", {
                  code: code,
                  map: {
                    mappings: ''
                  }
                });

              case 11:
                if (!(q.type === 'template')) {
                  _context.next = 16;
                  break;
                }

                debug("  > Compile --> ".concat(q.relative));
                output = compiler.compileTemplate({
                  code: source,
                  descriptor: cache[q.id].template
                }, q.id, _extends({}, vueOptions, {
                  scopeId: scopeId
                }));
                if (!output.map) output.map = {
                  mappings: ''
                };
                return _context.abrupt("return", output);

              case 16:
                if (!(q.type === 'style')) {
                  _context.next = 23;
                  break;
                }

                debug("  > Compile --> ".concat(q.relative));
                style = cache[q.id].styles[q.index];
                _context.next = 21;
                return compiler.compileStyle({
                  code: source,
                  descriptor: style
                }, q.id, _extends({}, vueOptions, {
                  scopeId: scopeId,
                  async: true
                }));

              case 21:
                result = _context.sent;
                return _context.abrupt("return", {
                  code: inlineStyle({
                    id: path.relative(cwd, q.id),
                    index: q.index,
                    code: result.code,
                    modules: result.modules,
                    map: result.map
                  }, style, _extends({}, vueOptions, {
                    inject: !opts.extract
                  })),
                  map: {
                    mappings: ''
                  }
                });

              case 23:
              case "end":
                return _context.stop();
            }
          }
        }, _callee, this);
      }));

      return function transform(_x, _x2) {
        return _transform.apply(this, arguments);
      };
    }()
  };
}

module.exports = vue;
